/*
 * File: Sudoku.java
 * -----------------
 * This program checks a Sudoku grid to see if it is valid.
 */

import acm.program.*;

public class Sudoku extends ConsoleProgram {
	
	private void doGrid(int[][] grid){
		printGrid(grid);
		println(checkSudokuSolution(grid));
	}

	public void run() {
		setFont("Courier-25");
		for(int i = 0; i < allGrids.length; i++){
			doGrid(allGrids[i]);
			readLine();
		}

	}
	
	private boolean checkSudokuSolution(int[][] grid){
		for(int i = 0; i < 9; i ++){
			//Get row i , col i, box i check them
			if(badGroupNine(getRow(grid,i))) return false;
			if(badGroupNine(getCol(grid,i))) return false;
			if(badGroupNine(getBox(grid,i))) return false;
		}
		return true;
	}
	
	private boolean badGroupNine(int[] group){
		boolean[] alreadySeen = new boolean[10];
		for(int i = 0; i < 9; i++){
			int value = group[i];
			if(alreadySeen[value]) return true;
			alreadySeen[value] = true;
		}
		return false;
	}
	
	private int[] getRow(int[][] grid, int row){
		return grid[row];
	}
	
	private int[] getCol(int[][] grid, int col){
		int[] column =  new int[9];
		for(int i = 0; i < 9; i ++){
			column[i] = grid[i][col];
		}
		return column;
	}
	
	private int[] getBox(int[][] grid, int index){
		int baseRow = index / 3 * 3;
		int baseCol = index % 3 * 3;
		int[] box = new int[9];
		for(int i = 0; i < 9; i ++){
			int row = baseRow + i/3;
			int col = baseCol + i%3;
			box[i] = grid[row][col];
		}
		return box;
	}
	
	private void printGrid(int[][] grid){
		println("");
		for(int i = 0; i < grid.length; i++){
			for(int j = 0; j < grid[0].length; j++){
				print(grid[i][j] + " ");
			}
			println("");
		}
	}


/* Test grid */
	private static final int[][] TEST_GRID = {
		{ 3, 9, 2, 4, 6, 5, 8, 1, 7 },
		{ 7, 4, 1, 8, 9, 3, 6, 2, 5 },
		{ 6, 8, 5, 2, 7, 1, 4, 3, 9 },
		{ 2, 5, 4, 1, 3, 8, 7, 9, 6 },
		{ 8, 3, 9, 6, 2, 7, 1, 5, 4 },
		{ 1, 7, 6, 9, 5, 4, 2, 8, 3 },
		{ 9, 6, 7, 5, 8, 2, 3, 4, 1 },
		{ 4, 2, 3, 7, 1, 9, 5, 6, 8 },
		{ 5, 1, 8, 3, 4, 6, 9, 7, 2 }
	};
	
	private static final int[][] BAD_GRID = {
		{ 3, 9, 3, 4, 6, 5, 8, 1, 7 },
		{ 7, 4, 1, 8, 9, 3, 6, 2, 5 },
		{ 6, 8, 5, 2, 7, 1, 4, 3, 9 },
		{ 2, 5, 4, 1, 3, 8, 7, 9, 6 },
		{ 8, 3, 9, 6, 2, 7, 1, 5, 4 },
		{ 1, 7, 6, 9, 5, 4, 2, 8, 3 },
		{ 9, 6, 7, 5, 8, 2, 3, 4, 1 },
		{ 4, 2, 3, 7, 1, 9, 5, 6, 8 },
		{ 5, 1, 8, 3, 4, 6, 9, 7, 2 }
	}; // 2 3s in row 1
	
	private static final int[][] BAD_GRID_2 = {
		{ 3, 9, 2, 4, 6, 5, 8, 1, 7 },
		{ 7, 4, 1, 8, 9, 3, 6, 2, 5 },
		{ 6, 8, 5, 2, 7, 1, 4, 3, 9 },
		{ 2, 5, 4, 1, 3, 8, 7, 9, 6 },
		{ 8, 3, 9, 7, 2, 7, 1, 5, 4 }, 
		{ 1, 7, 6, 9, 5, 4, 2, 8, 3 },
		{ 9, 6, 7, 5, 8, 2, 3, 4, 1 },
		{ 4, 2, 3, 7, 1, 9, 5, 6, 8 },
		{ 5, 1, 8, 3, 4, 6, 9, 7, 2 }
	}; // 2 7s in col 4
	
	private static final int[][] BAD_GRID_3 = {
		{ 3, 9, 2, 4, 6, 5, 8, 1, 7 },
		{ 7, 4, 1, 8, 9, 3, 6, 2, 5 },
		{ 6, 8, 5, 2, 7, 1, 4, 3, 9 },
		{ 2, 5, 4, 1, 3, 8, 7, 9, 6 },
		{ 8, 3, 9, 6, 2, 7, 1, 5, 4 }, 
		{ 1, 7, 6, 9, 5, 4, 2, 8, 3 },
		{ 9, 6, 7, 5, 8, 2, 3, 4, 1 },
		{ 4, 2, 3, 7, 1, 9, 5, 1, 8 },
		{ 5, 1, 8, 3, 4, 6, 9, 7, 2 }
	}; // 2 7s in col 4
	
	private static final int[][][] allGrids = { TEST_GRID, BAD_GRID, BAD_GRID_2, BAD_GRID_3 };

}